@@ -1,9 +1,11 @@
 import itertools
-from typing import Dict, List, Sequence
+from typing import Dict, List
 
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
+from torch.optim import Adam
+
 from ai_traineree import DEVICE
 from ai_traineree.agents import AgentBase
 from ai_traineree.agents.agent_utils import hard_update, soft_update
@@ -12,8 +14,8 @@ from ai_traineree.loggers import DataLogger
 from ai_traineree.networks.bodies import ActorBody, CriticBody
 from ai_traineree.networks.heads import CategoricalNet
 from ai_traineree.policies import MultivariateGaussianPolicy, MultivariateGaussianPolicySimple
+from ai_traineree.types.primitive import ActionType, DoneType, ObsType, RewardType
 from ai_traineree.utils import to_numbers_seq, to_tensor
-from torch.optim import Adam
 
 
 class D4PGAgent(AgentBase):
@@ -33,14 +35,14 @@ class D4PGAgent(AgentBase):
 
     name = "D4PG"
 
-    def __init__(self, state_size: int, action_size: int, hidden_layers: Sequence[int]=(128, 128), **kwargs):
+    def __init__(self, obs_size: int, action_size: int, **kwargs):
         """
         Parameters:
-            state_size (int): Number of input dimensions.
+            obs_size (int): Number of input dimensions.
             action_size (int): Number of output dimensions
-            hidden_layers (tuple of ints): Tuple defining hidden dimensions in fully connected nets. Default: (128, 128).
 
         Keyword parameters:
+            hidden_layers (tuple of ints): Shape of the hidden layers in fully connected network. Default: (128, 128).
             gamma (float): Discount value. Default: 0.99.
             tau (float): Soft-copy factor. Default: 0.02.
             actor_lr (float): Learning rate for the actor (policy). Default: 0.0003.
@@ -66,8 +68,11 @@ class D4PGAgent(AgentBase):
         """
         super().__init__(**kwargs)
         self.device = self._register_param(kwargs, "device", DEVICE)
-        self.state_size = state_size
+        self.obs_size = obs_size
         self.action_size = action_size
+        self._config['obs_size'] = self.obs_size
+        self._config['action_size'] = self.action_size
+        obs_shape = (obs_size,)
 
         self.num_atoms = int(self._register_param(kwargs, 'num_atoms', 51))
         v_min = float(self._register_param(kwargs, 'v_min', -10))
@@ -90,6 +95,7 @@ class D4PGAgent(AgentBase):
         self.warm_up = int(self._register_param(kwargs, 'warm_up', 0))
         self.update_freq = int(self._register_param(kwargs, 'update_freq', 1))
 
+        hidden_layers = to_numbers_seq(self._register_param(kwargs, 'hidden_layers', (128, 128)))
         self.actor_hidden_layers = to_numbers_seq(self._register_param(kwargs, 'actor_hidden_layers', hidden_layers))
         self.critic_hidden_layers = to_numbers_seq(self._register_param(kwargs, 'critic_hidden_layers', hidden_layers))
 
@@ -97,28 +103,32 @@ class D4PGAgent(AgentBase):
             std_init = float(self._register_param(kwargs, "std_init", 1.0))
             std_max = float(self._register_param(kwargs, "std_max", 2.0))
             std_min = float(self._register_param(kwargs, "std_min", 0.05))
-            self.policy = MultivariateGaussianPolicySimple(self.action_size, std_init=std_init, std_min=std_min, std_max=std_max, device=self.device)
+            self.policy = MultivariateGaussianPolicySimple(
+                self.action_size, std_init=std_init, std_min=std_min, std_max=std_max, device=self.device,
+            )
         else:
             self.policy = MultivariateGaussianPolicy(self.action_size, device=self.device)
 
         # This looks messy but it's not that bad. Actor, critic_net and Critic(critic_net). Then the same for `target_`.
         self.actor = ActorBody(
-            state_size, self.policy.param_dim*action_size, hidden_layers=self.actor_hidden_layers,
+            obs_shape, (self.policy.param_dim*action_size,), hidden_layers=self.actor_hidden_layers,
             gate_out=torch.tanh, device=self.device
         )
         critic_net = CriticBody(
-            state_size, action_size, out_features=self.num_atoms, hidden_layers=self.critic_hidden_layers, device=self.device
+            obs_shape, action_size, out_features=(self.num_atoms,),
+            hidden_layers=self.critic_hidden_layers, device=self.device
         )
         self.critic = CategoricalNet(
             num_atoms=self.num_atoms, v_min=v_min, v_max=v_max, net=critic_net, device=self.device
         )
 
         self.target_actor = ActorBody(
-            state_size, self.policy.param_dim*action_size, hidden_layers=self.actor_hidden_layers,
+            obs_shape, (self.policy.param_dim*action_size,), hidden_layers=self.actor_hidden_layers,
             gate_out=torch.tanh, device=self.device
         )
         target_critic_net = CriticBody(
-            state_size, action_size, out_features=self.num_atoms, hidden_layers=self.critic_hidden_layers, device=self.device
+            obs_shape, action_size, out_features=(self.num_atoms,),
+            hidden_layers=self.critic_hidden_layers, device=self.device
         )
         self.target_critic = CategoricalNet(
             num_atoms=self.num_atoms, v_min=v_min, v_max=v_max, net=target_critic_net, device=self.device
@@ -160,22 +170,22 @@ class D4PGAgent(AgentBase):
             self._loss_critic = value
 
     @torch.no_grad()
-    def act(self, state, epsilon: float=0.) -> List[float]:
+    def act(self, obs: ObsType, epsilon: float=0.) -> List[float]:
         """
-        Returns actions for given state as per current policy.
+        Returns actions for given observation as per current policy.
 
         Parameters:
-            state: Current available state from the environment.
+            obs: Current available observation from the environment.
             epislon: Epsilon value in the epislon-greedy policy.
 
         """
         actions = []
-        state = to_tensor(state).view(self.num_workers, self.state_size).float().to(self.device)
+        t_obs = to_tensor(obs).view(self.num_workers, self.obs_size).float().to(self.device)
         for worker in range(self.num_workers):
             if self._rng.random() < epsilon:
                 action = self.action_scale*(torch.rand(self.action_size) - 0.5)
             else:
-                action_seed = self.actor.act(state[worker].view(1, -1))
+                action_seed = self.actor.act(t_obs[worker].view(1, -1))
                 action_dist = self.policy(action_seed)
                 action = action_dist.sample()
                 action *= self.action_scale
@@ -185,13 +195,16 @@ class D4PGAgent(AgentBase):
         assert len(actions) == self.num_workers
         return actions
 
-    def step(self, states, actions, rewards, next_states, dones):
+    def step(
+        self, obss: List[ObsType], actions: List[ActionType], rewards: List[RewardType],
+        next_obss: List[ObsType], dones: List[DoneType],
+    ):
         self.iteration += 1
 
         # Delay adding to buffer to account for n_steps (particularly the reward)
         self.n_buffer.add(
-            state=torch.tensor(states).reshape(self.num_workers, self.state_size).float(),
-            next_state=torch.tensor(next_states).reshape(self.num_workers, self.state_size).float(),
+            state=torch.tensor(obss).reshape(self.num_workers, self.obs_size).float(),
+            next_state=torch.tensor(next_obss).reshape(self.num_workers, self.obs_size).float(),
             action=torch.tensor(actions).reshape(self.num_workers, self.action_size).float(),
             reward=torch.tensor(rewards).reshape(self.num_workers, 1),
             done=torch.tensor(dones).reshape(self.num_workers, 1),
@@ -227,16 +240,16 @@ class D4PGAgent(AgentBase):
         next_actions = self.policy(next_action_seeds).sample()
         assert next_actions.shape == (self.batch_size, self.action_size)
 
-        target_value_dist_estimate = self.target_critic.act(states, next_actions)
-        assert target_value_dist_estimate.shape == (self.batch_size, 1, self.num_atoms)
-        target_value_dist_estimate = target_value_dist_estimate.squeeze()
-        assert target_value_dist_estimate.shape == (self.batch_size, self.num_atoms)
+        target_value_dist_est = self.target_critic.act(states, next_actions)
+        assert target_value_dist_est.shape == (self.batch_size, 1, self.num_atoms)
+        target_value_dist_est = target_value_dist_est.squeeze()
+        assert target_value_dist_est.shape == (self.batch_size, self.num_atoms)
 
         discount = self.gamma ** self.n_steps
-        target_value_projected = self.target_critic.dist_projection(rewards, 1 - dones, discount, target_value_dist_estimate)
+        target_value_projected = self.target_critic.dist_projection(rewards, 1 - dones, discount, target_value_dist_est)
         assert target_value_projected.shape == (self.batch_size, self.num_atoms)
 
-        target_value_dist = F.softmax(target_value_dist_estimate, dim=-1).detach()
+        target_value_dist = F.softmax(target_value_dist_est, dim=-1).detach()
         assert target_value_dist.shape == (self.batch_size, self.num_atoms)
 
         # Comparing Q_w with Q_w'
@@ -267,9 +280,9 @@ class D4PGAgent(AgentBase):
         # No need for size assertion since .view() has explicit sizes
         rewards = to_tensor(experiences['reward']).view(self.batch_size, 1).float().to(self.device)
         dones = to_tensor(experiences['done']).view(self.batch_size, 1).type(torch.int).to(self.device)
-        states = to_tensor(experiences['state']).view(self.batch_size, self.state_size).float().to(self.device)
+        states = to_tensor(experiences['state']).view(self.batch_size, self.obs_size).float().to(self.device)
         actions = to_tensor(experiences['action']).view(self.batch_size, self.action_size).to(self.device)
-        next_states = to_tensor(experiences['next_state']).view(self.batch_size, self.state_size).float().to(self.device)
+        next_states = to_tensor(experiences['next_state']).view(self.batch_size, self.obs_size).float().to(self.device)
 
         indices = None
         if hasattr(self.buffer, 'priority_update'):  # When using PER buffer
