@@ -1,11 +1,11 @@
 import itertools
-import operator
-from functools import reduce
 from typing import Dict, List, Tuple, Union
 
 import numpy as np
 import torch
 import torch.nn as nn
+from torch import Tensor, optim
+
 from ai_traineree import DEVICE
 from ai_traineree.agents import AgentBase
 from ai_traineree.agents.agent_utils import hard_update, soft_update
@@ -14,9 +14,8 @@ from ai_traineree.loggers import DataLogger
 from ai_traineree.networks.bodies import ActorBody, CriticBody
 from ai_traineree.networks.heads import DoubleCritic
 from ai_traineree.policies import GaussianPolicy, MultivariateGaussianPolicySimple
-from ai_traineree.types import FeatureType
+from ai_traineree.types.primitive import ActionType, DoneType, ObsType, RewardType
 from ai_traineree.utils import to_numbers_seq, to_tensor
-from torch import Tensor, optim
 
 
 class SACAgent(AgentBase):
@@ -32,31 +31,38 @@ class SACAgent(AgentBase):
 
     name = "SAC"
 
-    def __init__(self, in_features: FeatureType, action_size: int, **kwargs):
+    def __init__(self, obs_size: int, action_size: int, **kwargs):
         """
         Parameters:
-            hidden_layers: (default: (128, 128)) Shape of the hidden layers that are fully connected networks.
-            gamma: (default: 0.99) Discount value.
-            tau: (default: 0.02) Soft copy fraction.
-            batch_size: (default 64) Number of samples in a batch.
-            buffer_size: (default: 1e6) Size of the prioritized experience replay buffer.
-            warm_up: (default: 0) Number of samples that needs to be observed before starting to learn.
-            update_freq: (default: 1) Number of samples between policy updates.
-            number_updates: (default: 1) Number of times of batch sampling/training per `update_freq`.
-            alpha: (default: 0.2) Weight of log probs in value function.
-            alpha_lr: (default: None) If provided, it will add alpha as a training parameters and `alpha_lr` is its learning rate.
-            action_scale: (default: 1.) Scale for returned action values.
-            max_grad_norm_alpha: (default: 1.) Gradient clipping for the alpha.
-            max_grad_norm_actor: (default 10.) Gradient clipping for the actor.
-            max_grad_norm_critic: (default: 10.) Gradient clipping for the critic.
-            device: Defaults to CUDA if available.
+            obs_size (int): Dimension of the observation.
+            action_size (int): Dimension of expected action.
+
+        Keyword parameters:
+            hidden_layers (tuple of ints): Shape of the hidden layers in fully connected network. Default: (128, 128).
+            gamma (float):  Discount value. Default: 0.99.
+            tau (float): Soft copy fraction. Default: 0.02.
+            batch_size (int): Number of samples in a batch. Default: 64.
+            buffer_size (int): Size of the prioritized experience replay buffer. Default: 1e6.
+            warm_up: (default: 0) Number of samples that needs to be observed before starting to learn. Default: 0.
+            update_freq (int): Number of samples between policy updates. Default: 1.
+            number_updates (int):  Number of times of batch sampling/training per `update_freq`. Default: 1.
+            alpha (float): Weight of log probs in value function. Default: 0.2.
+            alpha_lr (Optional float): If not None, it adds alpha as a training parameters with `alpha_lr` as its
+                learning rate. Default: None.
+            action_scale (float):  Scale for returned action values. Default: 1.
+            max_grad_norm_alpha (float): Gradient clipping for the alpha. Default: 1.
+            max_grad_norm_actor (float): Gradient clipping for the actor. Default: 10.
+            max_grad_norm_critic (float):  Gradient clipping for the critic. Default: 10.
+            device: Defaults to CUDA if available. Default: CUDA if available.
 
         """
         super().__init__(**kwargs)
         self.device = kwargs.get("device", DEVICE)
-        self.in_features: Tuple[int] = (in_features,) if isinstance(in_features, int) else tuple(in_features)
-        self.state_size: int = in_features if isinstance(in_features, int) else reduce(operator.mul, in_features)
+        self.obs_size = obs_size
         self.action_size = action_size
+        self._config['obs_size'] = self.obs_size
+        self._config['action_size'] = self.action_size
+        obs_shape = (obs_size,)
 
         self.gamma: float = float(self._register_param(kwargs, 'gamma', 0.99))
         self.tau: float = float(self._register_param(kwargs, 'tau', 0.02))
@@ -82,15 +88,18 @@ class SACAgent(AgentBase):
         self.simple_policy = bool(self._register_param(kwargs, "simple_policy", False))
         if self.simple_policy:
             self.policy = MultivariateGaussianPolicySimple(self.action_size, **kwargs)
-            self.actor = ActorBody(self.state_size, self.policy.param_dim*self.action_size, hidden_layers=actor_hidden_layers, device=self.device)
+            self.actor = ActorBody(
+                obs_shape, (self.policy.param_dim*self.action_size,), hidden_layers=actor_hidden_layers, device=self.device)
         else:
-            self.policy = GaussianPolicy(actor_hidden_layers[-1], self.action_size, out_scale=self.action_scale, device=self.device)
-            self.actor = ActorBody(self.state_size, actor_hidden_layers[-1], hidden_layers=actor_hidden_layers[:-1], device=self.device)
+            self.policy = GaussianPolicy(
+                (actor_hidden_layers[-1],), (self.action_size,), out_scale=self.action_scale, device=self.device)
+            self.actor = ActorBody(
+                obs_shape, (actor_hidden_layers[-1],), hidden_layers=actor_hidden_layers[:-1], device=self.device)
 
         self.double_critic = DoubleCritic(
-            self.in_features, self.action_size, CriticBody, hidden_layers=critic_hidden_layers, device=self.device)
+            obs_shape, self.action_size, CriticBody, hidden_layers=critic_hidden_layers, device=self.device)
         self.target_double_critic = DoubleCritic(
-            self.in_features, self.action_size, CriticBody, hidden_layers=critic_hidden_layers, device=self.device)
+            obs_shape, self.action_size, CriticBody, hidden_layers=critic_hidden_layers, device=self.device)
 
         # Target sequence initiation
         hard_update(self.target_double_critic, self.double_critic)
@@ -118,8 +127,8 @@ class SACAgent(AgentBase):
         # Breath, my child.
         self.iteration = 0
 
-        self._loss_actor = float('inf')
-        self._loss_critic = float('inf')
+        self._loss_actor = float('nan')
+        self._loss_critic = float('nan')
         self._metrics: Dict[str, Union[float, Dict[str, float]]] = {}
 
     @property
@@ -127,7 +136,7 @@ class SACAgent(AgentBase):
         return self.log_alpha.exp()
 
     @property
-    def loss(self):
+    def loss(self) -> Dict[str, float]:
         return {'actor': self._loss_actor, 'critic': self._loss_critic}
 
     @loss.setter
@@ -158,21 +167,32 @@ class SACAgent(AgentBase):
         }
 
     @torch.no_grad()
-    def act(self, state, epsilon: float=0.0, deterministic=False) -> List[float]:
+    def act(self, obs: ObsType, epsilon: float=0.0, deterministic: bool=False) -> List[float]:
+        """Acting on the observations. Returns action.
+
+        Parameters:
+            obs (array_like): current state
+            eps (float): epsilon, for epsilon-greedy action selection
+            deterministic (optional bool): Whether to use deterministic policy.
+
+        Returns:
+            action: (list float) Action values.
+
+        """
         if self.iteration < self.warm_up or self._rng.random() < epsilon:
             random_action = torch.rand(self.action_size) * (self.action_max + self.action_min) + self.action_min
             return random_action.cpu().tolist()
 
-        state = to_tensor(state).view(1, self.state_size).float().to(self.device)
-        proto_action = self.actor(state)
+        t_obs = to_tensor(obs).view(1, self.obs_size).float().to(self.device)
+        proto_action = self.actor(t_obs)
         action = self.policy(proto_action, deterministic)
 
         return action.flatten().tolist()
 
-    def step(self, state, action, reward, next_state, done):
+    def step(self, obs: ObsType, action: ActionType, reward: RewardType, next_obs: ObsType, done: DoneType):
         self.iteration += 1
         self.memory.add(
-            state=state, action=action, reward=reward, next_state=next_state, done=done,
+            state=obs, action=action, reward=reward, next_state=next_obs, done=done,
         )
 
         if self.iteration < self.warm_up:
@@ -204,20 +224,20 @@ class SACAgent(AgentBase):
 
         Q1_diff = Q1_expected - Q_target
         error_1 = Q1_diff.pow(2)
-        mse_loss_1 = error_1.mean()
+        mse_loss_1: Tensor = error_1.mean()
         self._metrics['value/critic1'] = {'mean': float(Q1_expected.mean()), 'std': float(Q1_expected.std())}
         self._metrics['value/critic1_lse'] = float(mse_loss_1.item())
 
         Q2_diff = Q2_expected - Q_target
         error_2 = Q2_diff.pow(2)
-        mse_loss_2 = error_2.mean()
+        mse_loss_2: Tensor = error_2.mean()
         self._metrics['value/critic2'] = {'mean': float(Q2_expected.mean()), 'std': float(Q2_expected.std())}
         self._metrics['value/critic2_lse'] = float(mse_loss_2.item())
 
         Q_diff = Q1_expected - Q2_expected
         self._metrics['value/Q_diff'] = {'mean': float(Q_diff.mean()), 'std': float(Q_diff.std())}
 
-        error = torch.min(error_1, error_2)
+        error: Tensor = torch.min(error_1, error_2)
         loss = mse_loss_1 + mse_loss_2
         return loss, error
 
@@ -248,8 +268,8 @@ class SACAgent(AgentBase):
 
         rewards = to_tensor(samples['reward']).float().to(self.device).view(self.batch_size, 1)
         dones = to_tensor(samples['done']).int().to(self.device).view(self.batch_size, 1)
-        states = to_tensor(samples['state']).float().to(self.device).view(self.batch_size, self.state_size)
-        next_states = to_tensor(samples['next_state']).float().to(self.device).view(self.batch_size, self.state_size)
+        states = to_tensor(samples['state']).float().to(self.device).view(self.batch_size, self.obs_size)
+        next_states = to_tensor(samples['next_state']).float().to(self.device).view(self.batch_size, self.obs_size)
         actions = to_tensor(samples['action']).to(self.device).view(self.batch_size, self.action_size)
 
         # Critic (value) update
@@ -259,7 +279,7 @@ class SACAgent(AgentBase):
             value_loss.backward()
             nn.utils.clip_grad_norm_(self.critic_params, self.max_grad_norm_critic)
             self.critic_optimizer.step()
-            self._loss_critic = value_loss.item()
+            self._loss_critic = float(value_loss.item())
 
         # Actor (policy) update
         for _ in range(self.actor_number_updates):
@@ -268,7 +288,7 @@ class SACAgent(AgentBase):
             policy_loss.backward()
             nn.utils.clip_grad_norm_(self.actor_params, self.max_grad_norm_actor)
             self.actor_optimizer.step()
-            self._loss_actor = policy_loss.item()
+            self._loss_actor = float(policy_loss.item())
 
         if hasattr(self.memory, 'priority_update'):
             assert any(~torch.isnan(error))
