@@ -3,6 +3,9 @@ from typing import Dict, List
 import numpy as np
 import torch
 import torch.nn as nn
+from torch.nn.functional import mse_loss
+from torch.optim import AdamW
+
 from ai_traineree import DEVICE
 from ai_traineree.agents import AgentBase
 from ai_traineree.agents.agent_utils import hard_update, soft_update
@@ -11,9 +14,8 @@ from ai_traineree.loggers import DataLogger
 from ai_traineree.networks.bodies import ActorBody, CriticBody
 from ai_traineree.networks.heads import DoubleCritic
 from ai_traineree.noise import OUProcess
+from ai_traineree.types.primitive import ActionType, DoneType, ObsType, RewardType
 from ai_traineree.utils import to_numbers_seq, to_tensor
-from torch.nn.functional import mse_loss
-from torch.optim import AdamW
 
 
 class TD3Agent(AgentBase):
@@ -26,16 +28,16 @@ class TD3Agent(AgentBase):
 
     name = "TD3"
 
-    def __init__(self, state_size: int, action_size: int, noise_scale: float=0.2, noise_sigma: float=0.1, **kwargs):
+    def __init__(self, obs_size: int, action_size: int, noise_scale: float=0.2, noise_sigma: float=0.1, **kwargs):
         """
         Parameters:
-            state_size (int): Number of input dimensions.
+            obs_size (int): Number of input dimensions.
             action_size (int): Number of output dimensions
             noise_scale (float): Added noise amplitude. Default: 0.2.
             noise_sigma (float): Added noise variance. Default: 0.1.
 
         Keyword parameters:
-            hidden_layers (tuple of ints): Tuple defining hidden dimensions in fully connected nets. Default: (128, 128).
+            hidden_layers (tuple of ints): Shape of the hidden layers in fully connected network. Default: (128, 128).
             actor_lr (float): Learning rate for the actor (policy). Default: 0.003.
             critic_lr (float): Learning rate for the critic (value function). Default: 0.003.
             gamma (float): Discount value. Default: 0.99.
@@ -58,14 +60,17 @@ class TD3Agent(AgentBase):
         self.device = self._register_param(kwargs, "device", DEVICE)  # Default device is CUDA if available
 
         # Reason sequence initiation.
-        self.state_size = state_size
+        self.obs_size = obs_size
         self.action_size = action_size
+        self._config['obs_size'] = self.obs_size
+        self._config['action_size'] = self.action_size
+        obs_shape, action_shape = (obs_size,), (action_size,)
 
         hidden_layers = to_numbers_seq(self._register_param(kwargs, 'hidden_layers', (128, 128)))
-        self.actor = ActorBody(state_size, action_size, hidden_layers=hidden_layers).to(self.device)
-        self.critic = DoubleCritic(state_size, action_size, CriticBody, hidden_layers=hidden_layers).to(self.device)
-        self.target_actor = ActorBody(state_size, action_size, hidden_layers=hidden_layers).to(self.device)
-        self.target_critic = DoubleCritic(state_size, action_size, CriticBody, hidden_layers=hidden_layers).to(self.device)
+        self.actor = ActorBody(obs_shape, action_shape, hidden_layers=hidden_layers).to(self.device)
+        self.critic = DoubleCritic(obs_shape, action_size, CriticBody, hidden_layers=hidden_layers).to(self.device)
+        self.target_actor = ActorBody(obs_shape, action_shape, hidden_layers=hidden_layers).to(self.device)
+        self.target_critic = DoubleCritic(obs_shape, action_size, CriticBody, hidden_layers=hidden_layers).to(self.device)
 
         # Noise sequence initiation
         # self.noise = GaussianNoise(shape=(action_size,), mu=1e-8, sigma=noise_sigma, scale=noise_scale, device=device)
@@ -101,8 +106,8 @@ class TD3Agent(AgentBase):
         # Breath, my child.
         self.reset_agent()
         self.iteration = 0
-        self._loss_actor = 0.
-        self._loss_critic = 0.
+        self._loss_actor = float('nan')
+        self._loss_critic = float('nan')
 
     @property
     def loss(self) -> Dict[str, float]:
@@ -123,7 +128,8 @@ class TD3Agent(AgentBase):
         self.target_actor.reset_parameters()
         self.target_critic.reset_parameters()
 
-    def act(self, state, epsilon: float=0.0, training_mode=True) -> List[float]:
+    @torch.no_grad()
+    def act(self, obs: ObsType, epsilon: float=0.0, training_mode: bool=True) -> List[float]:
         """
         Agent acting on observations.
 
@@ -134,22 +140,21 @@ class TD3Agent(AgentBase):
             rnd_actions = torch.rand(self.action_size)*(self.action_max - self.action_min) - self.action_min
             return rnd_actions.tolist()
 
-        with torch.no_grad():
-            state = to_tensor(state).float().to(self.device)
-            action = self.actor(state)
-            if training_mode:
-                action += self.noise.sample()
-            return (self.action_scale*torch.clamp(action, self.action_min, self.action_max)).tolist()
+        t_obs = to_tensor(obs).float().to(self.device)
+        action = self.actor(t_obs)
+        if training_mode:
+            action += self.noise.sample()
+        return (self.action_scale*torch.clamp(action, self.action_min, self.action_max)).tolist()
 
-    def target_act(self, staten, noise: float=0.0):
-        with torch.no_grad():
-            staten = to_tensor(staten).float().to(self.device)
-            action = self.target_actor(staten) + noise*self.noise.sample()
-            return torch.clamp(action, self.action_min, self.action_max).cpu().numpy().astype(np.float32)
+    @torch.no_grad()
+    def target_act(self, obs: ObsType, noise: float=0.0):
+        t_obs = to_tensor(obs).float().to(self.device)
+        action = self.target_actor(t_obs) + noise*self.noise.sample()
+        return torch.clamp(action, self.action_min, self.action_max).cpu().numpy().astype(np.float32)
 
-    def step(self, state, action, reward, next_state, done):
+    def step(self, obs: ObsType, action: ActionType, reward: RewardType, next_obs: ObsType, done: DoneType):
         self.iteration += 1
-        self.buffer.add(state=state, action=action, reward=reward, next_state=next_state, done=done)
+        self.buffer.add(state=obs, action=action, reward=reward, next_state=next_obs, done=done)
 
         if (self.iteration % self.noise_reset_freq) == 0:
             self.noise.reset_states()
