@@ -164,9 +164,9 @@ class EnvRunner:
         To help debugging one can set the `gif_every_episodes` to a positive integer which relates to how often a gif
         of the episode evaluation is written to the disk.
 
-        Every `checkpoint_every` (default: 200) iterations the Runner will store current state of the runner and the agent.
-        These states can be used to resume previous run. By default the runner checks whether there is ongoing run for
-        the combination of the environment and the agent.
+        Every `checkpoint_every` (default: 200) iterations the Runner will store current state of the runner
+        and the agent. These states can be used to resume previous run. By default the runner checks whether
+        there is ongoing run for the combination of the environment and the agent.
 
         Parameters:
             reward_goal: Goal to achieve on the average reward.
@@ -538,7 +538,7 @@ class MultiSyncEnvRunner:
         scores = np.zeros(self.task_num)
         iterations = np.zeros(self.task_num)
 
-        states = np.empty((self.num_processes, self.tasks[0].state_size), dtype=np.float32)
+        states = np.empty((self.num_processes, self.tasks[0].obs_size), dtype=np.float32)
         next_states = states.copy()
         actions = np.empty((len(self.tasks), self.tasks[0].action_size), dtype=np.float32)
         dones = np.empty(len(self.tasks))
