@@ -14,13 +14,17 @@ Heads are "special" in that each is built on networks/brains and will likely nee
 some special pipeping when attaching to your agent.
 """
 from functools import lru_cache, reduce
+from operator import mul
 from typing import Callable, List, Optional, Sequence
 
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
+
 from ai_traineree.networks import NetworkType, NetworkTypeClass
 from ai_traineree.networks.bodies import FcNet, NoisyNet
+from ai_traineree.types.primitive import FeatureType
+from ai_traineree.utils import to_numbers_seq
 
 
 class NetChainer(NetworkType):
@@ -61,8 +65,10 @@ class DoubleCritic(NetworkType):
     def __init__(self, in_features: Sequence[int], action_size: int, body_cls: NetworkTypeClass, **kwargs):
         super(DoubleCritic, self).__init__()
         hidden_layers = kwargs.pop("hidden_layers", (200, 200))
-        self.critic_1 = body_cls(in_features=in_features, action_size=action_size, hidden_layers=hidden_layers, **kwargs)
-        self.critic_2 = body_cls(in_features=in_features, action_size=action_size, hidden_layers=hidden_layers, **kwargs)
+        self.critic_1 = body_cls(
+            in_features=in_features, inj_action_size=action_size, hidden_layers=hidden_layers, **kwargs)
+        self.critic_2 = body_cls(
+            in_features=in_features, inj_action_size=action_size, hidden_layers=hidden_layers, **kwargs)
 
     def reset_parameters(self):
         self.critic_1.reset_parameters()
@@ -77,28 +83,37 @@ class DoubleCritic(NetworkType):
 
 class DuelingNet(NetworkType):
     def __init__(
-        self, input_shape: Sequence[int], output_shape: Sequence[int], hidden_layers: Sequence[int],
+        self, in_features: Sequence[int], out_features: Sequence[int], hidden_layers: Sequence[int],
         net_fn: Optional[Callable[..., NetworkType]]=None,
         net_class: Optional[NetworkTypeClass]=None,
         **kwargs
     ):
         """
         Parameters:
-            input_shape (Tuple of ints): Shape of the input. Even in case when input is 1D, a single item tuple is expected, e.g. (4,).
-            output_shape (Tuple of ints): Shape of the output. Same as with the `input_shape`.
+            in_features (tuple of ints): Dimension of the input features.
+            out_features (tuple of ints): Dimension of critic's action. Default: (1,).
+            hidden_layers (tuple of ints): Shape of the hidden layers.
+            net_fn (optional func):
+            net_class (optional class)
+        
+        Keyword parameters:
+            device: Device where to allocate memory. CPU or CUDA. Default CUDA if available.
+
         """
         super(DuelingNet, self).__init__()
         device = kwargs.get("device")
         # We only care about the leading size, e.g. (4,) -> 4
         if net_fn is not None:
-            self.value_net = net_fn(input_shape, (1,), hidden_layers=hidden_layers)
-            self.advantage_net = net_fn(input_shape, output_shape, hidden_layers=hidden_layers)
+            self.value_net = net_fn(in_features, (1,), hidden_layers=hidden_layers)
+            self.advantage_net = net_fn(in_features, out_features, hidden_layers=hidden_layers)
         elif net_class is not None:
-            self.value_net = net_class(input_shape, (1,), hidden_layers=hidden_layers, device=device)
-            self.advantage_net = net_class(input_shape, output_shape, hidden_layers=hidden_layers, device=device)
+            self.value_net = net_class(in_features, (1,), hidden_layers=hidden_layers, device=device)
+            self.advantage_net = net_class(in_features, out_features, hidden_layers=hidden_layers, device=device)
         else:
-            self.value_net = FcNet(input_shape, (1,), hidden_layers=hidden_layers, gate_out=None, device=device)
-            self.advantage_net = FcNet(input_shape, output_shape, hidden_layers=hidden_layers, gate_out=None, device=device)
+            self.value_net = FcNet(
+                in_features, (1,), hidden_layers=hidden_layers, gate_out=nn.Identity(), device=device)
+            self.advantage_net = FcNet(
+                in_features, out_features, hidden_layers=hidden_layers, gate_out=nn.Identity(), device=device)
 
     def reset_parameters(self) -> None:
         self.value_net.reset_parameters()
@@ -135,8 +150,8 @@ class CategoricalNet(NetworkType):
         num_atoms: int=21,
         v_min: float=-20.,
         v_max: float=20.,
-        state_size: Optional[int]=None,
-        action_size: Optional[int]=None,
+        in_features: Optional[FeatureType]=None,
+        out_features: Optional[FeatureType]=None,
         hidden_layers: Sequence[int]=(200, 200),
         net: Optional[NetworkType]=None,
         device: Optional[torch.device]=None,
@@ -147,12 +162,12 @@ class CategoricalNet(NetworkType):
             v_min: Minimum (edge) value of the shifted distribution.
             v_max: Maximum (edge) value of the shifted distribution.
             net: (Optional) A network used for estimation. If `net` is proved then `hidden_layers` has no effect.
-            state_size: Size of the state.
+            obs_space: Size of the observation.
             action_size: Length of the output.
             hidden_layers: Shape of the hidden layers that are fully connected networks.
 
-        *Note* that either `net` or both (`state_size`, `action_size`) need to be not None.
-        If `state_size` and `action_size` are provided then the default net is created as
+        *Note* that either `net` or both (`obs_space`, `action_size`) need to be not None.
+        If `obs_space` and `action_size` are provided then the default net is created as
         fully connected network with `hidden_layers` size.
 
         """
@@ -166,13 +181,16 @@ class CategoricalNet(NetworkType):
 
         if net is not None:
             self.net = net
-        elif state_size is not None and action_size is not None:
-            self.net = FcNet(state_size, action_size*self.num_atoms, hidden_layers=hidden_layers, device=self.device)
+        elif in_features is not None and out_features is not None:
+            assert len(out_features) == 1, "Expecting single dimension for output features"
+            _out_features = (out_features[0]*self.num_atoms, )
+            self.net = FcNet(in_features, _out_features, hidden_layers=hidden_layers, device=self.device)
         else:
-            raise ValueError("CategoricalNet needs to be instantiated either with `net` or (`state_size` and `action_size`)")
+            raise ValueError("CategoricalNet needs to be instantiated either with `net` or (`obs_space` and `action_size`)")
 
+        assert len(self.net.out_features) == 1, "Expecting single dimension for output features"
         self.in_featores = self.net.in_features
-        self.out_features = (self.net.out_features // self.num_atoms, self.num_atoms)
+        self.out_features = (self.net.out_features[0] // self.num_atoms, self.num_atoms)
         self.to(device=device)
 
     def reset_paramters(self):
@@ -240,18 +258,17 @@ class RainbowNet(NetworkType, nn.Module):
     """
     def __init__(
         self,
-        input_shape: Sequence[int],
-        output_shape: Sequence[int],
-        hidden_layers=(200, 200),
+        in_features: FeatureType,
+        out_features: FeatureType,
         **kwargs
     ):
         """
         Parameters
-            input_shape (tuple of ints): Shape of the single input.
-            output_shape (tuple of ints): Shape of the expected output.
-            hidden_layers (tuple of ints): Shape of fully connected networks. Default: (200, 200).
+            in_features (tuple of ints): Shape of the input.
+            out_features (tuple of ints): Shape of the expected output.
 
         Keyword parameters:
+            hidden_layers (tuple of ints): Shape of fully connected networks. Default: (200, 200).
             num_atoms (int): Number of atoms used in estimating distribution. Default: 21.
             v_min (float): Value distribution minimum (left most) value. Default -10.
             v_max (float): Value distribution maximum (right most) value. Default 10.
@@ -264,12 +281,11 @@ class RainbowNet(NetworkType, nn.Module):
         self.device = device = kwargs.get("device", None)
 
         self.pre_network = None
-        in_features = input_shape[0]
-        out_features = output_shape[0]
         if 'pre_network_fn' in kwargs:
-            self.pre_network = kwargs.get("pre_network_fn")(in_features=input_shape)
+            self.pre_network = kwargs.get("pre_network_fn")(in_features=in_features)
             self.pre_netowrk_params = self.pre_network.parameters()  # Registers pre_network's parameters to this module
-            in_features = self.pre_network.out_features
+            pof = self.pre_network.out_features
+            in_features = (pof,) if isinstance(pof, int) else pof
 
         self.v_min = float(kwargs.get("v_min", -10))
         self.v_max = float(kwargs.get("v_max", 10))
@@ -277,15 +293,21 @@ class RainbowNet(NetworkType, nn.Module):
         self.z_atoms = torch.linspace(self.v_min, self.v_max, self.num_atoms, device=self.device)
         self.z_delta = self.z_atoms[1] - self.z_atoms[0]
 
+        in_size, out_size = reduce(mul, in_features), reduce(mul, out_features)
+        hidden_layers = to_numbers_seq(kwargs.get('hidden_layers', (128, 128)))
         self.noisy = kwargs.get("noisy", False)
         if self.noisy:
-            self.value_net = NoisyNet(in_features, num_atoms, hidden_layers=hidden_layers, device=device)
-            self.advantage_net = NoisyNet(in_features, out_features*num_atoms, hidden_layers=hidden_layers, device=device)
+            self.value_net = NoisyNet((in_size,), out_features=(num_atoms,), hidden_layers=hidden_layers, device=device)
+            self.advantage_net = NoisyNet((in_size,), out_size*num_atoms, hidden_layers=hidden_layers, device=device)
         else:
-            self.value_net = FcNet(in_features, num_atoms, hidden_layers=hidden_layers, gate_out=None, device=device)
-            self.advantage_net = FcNet((in_features,), (out_features*num_atoms,), hidden_layers=hidden_layers, gate_out=None, device=device)
+            self.value_net = FcNet(in_features, out_features=(num_atoms,), hidden_layers=hidden_layers, device=device)
+            self.advantage_net = FcNet(in_features, (out_size*num_atoms,), hidden_layers=hidden_layers, device=device)
 
-        self.in_features = in_features if self.pre_network is None else self.pre_network.in_features
+        if self.pre_network is not None:
+            pif = self.pre_network.in_features
+            self.in_features = (pif,) if isinstance(pif, int) else pif
+        else:
+            self.in_features = in_features
         self.out_features = out_features
         self.to(device=self.device)
 
@@ -311,7 +333,7 @@ class RainbowNet(NetworkType, nn.Module):
             if self.pre_network is not None:
                 x = self.pre_network(x)
             value = self.value_net.act(x).view(-1, 1, self.num_atoms)
-            advantage = self.advantage_net.act(x).view(-1, self.out_features, self.num_atoms)
+            advantage = self.advantage_net.act(x).view((-1,) + self.out_features + (self.num_atoms,))
             q = value + advantage - advantage.mean(1, keepdim=True)
             # Doc: It's computationally quicker than log(softmax) and more stable
             out = F.softmax(q, dim=-1) if not log_prob else F.log_softmax(q, dim=-1)
@@ -328,7 +350,7 @@ class RainbowNet(NetworkType, nn.Module):
         if self.pre_network is not None:
             x = self.pre_network(x)
         value = self.value_net(x).view((-1, 1, self.num_atoms))
-        advantage = self.advantage_net(x).view(-1, self.out_features, self.num_atoms)
+        advantage = self.advantage_net(x).view((-1,) + self.out_features + (self.num_atoms,))
         q = value + advantage - advantage.mean(1, keepdim=True)
         if log_prob:
             # Doc: It's computationally quicker than log(softmax) and more stable
