@@ -34,7 +34,7 @@ config = {
     "actor_lr": 3e-4,
     "actor_betas": (0.9, 0.999),
 }
-agent = Agent(task.state_size, task.action_size, hidden_layers=(100, 100), **config)
+agent = Agent(task.obs_size, task.action_size, hidden_layers=(100, 100), **config)
 env_runner = EnvRunner(task, agent, max_iterations=2000, data_logger=data_logger)
 # env_runner.interact_episode(render=True)
 scores = env_runner.run(300, 1000, log_episode_freq=1, gif_every_episodes=500, force_new=True)
